@@ -15,8 +15,8 @@
 import re
 import time
 
-from oslo.concurrency import processutils
-from oslo.utils import strutils
+from oslo_concurrency import processutils
+from oslo_utils import strutils
 
 from proliantutils import exception
 from proliantutils.hpssa import constants
@@ -58,7 +58,7 @@ def _get_key_value(string):
 def _get_dict(lines, start_index, indentation):
     """Recursive function for parsing hpssacli output."""
 
-    info = dict()
+    info = {}
     current_item = None
 
     i = start_index
@@ -69,7 +69,7 @@ def _get_dict(lines, start_index, indentation):
 
         if current_line_indentation == indentation:
             current_item = current_line.lstrip(' ')
-            info[current_item] = dict()
+            info[current_item] = {}
             i = i + 1
             continue
 
@@ -111,7 +111,7 @@ def _convert_to_dict(stdout):
     """
 
     lines = stdout.split("\n")
-    lines = filter(None, lines)
+    lines = list(filter(None, lines))
     info_dict, j = _get_dict(lines, 0, 0)
     return info_dict
 
@@ -231,7 +231,7 @@ class Server(object):
         raid_info = _convert_to_dict(config)
         self.controllers = []
 
-        for key, value in raid_info.iteritems():
+        for key, value in raid_info.items():
             self.controllers.append(Controller(key, value, self))
 
         self.last_updated = time.time()
@@ -312,8 +312,8 @@ class Controller(object):
         self.unassigned_physical_drives = []
         self.raid_arrays = []
 
-        unassigned_drives = properties.get('unassigned', dict())
-        for key, value in unassigned_drives.iteritems():
+        unassigned_drives = properties.get('unassigned', {})
+        for key, value in unassigned_drives.items():
             self.unassigned_physical_drives.append(PhysicalDrive(key,
                                                                  value,
                                                                  self))
@@ -485,8 +485,9 @@ class LogicalDrive(object):
         # It requires space to be stripped.
         size = self.properties['Size'].replace(' ', '')
         try:
-            self.size_gb = (strutils.string_to_bytes(size, return_int=True) /
-                            (1024*1024*1024))
+            self.size_gb = int(strutils.string_to_bytes(size,
+                                                        return_int=True) /
+                               (1024*1024*1024))
         except ValueError:
             msg = ("hpssacli returned unknown size '%(size)s' for logical "
                    "disk '%(logical_disk)s' of RAID array '%(array)s' in "
@@ -545,8 +546,9 @@ class PhysicalDrive:
         # (like 500MB, 25GB) unit of storage space to bytes (Integer value).
         # It requires space to be stripped.
         try:
-            self.size_gb = (strutils.string_to_bytes(size, return_int=True) /
-                            (1024*1024*1024))
+            self.size_gb = int(strutils.string_to_bytes(size,
+                                                        return_int=True) /
+                               (1024*1024*1024))
         except ValueError:
             msg = ("hpssacli returned unknown size '%(size)s' for physical "
                    "disk '%(physical_disk)s' of controller "
